/*
  @author: asukaze
  @url: http://www.asukaze.net/
  @title: Rotate Documents
  @description: Rotate HTML Documents.
  @version: 0.3.0
  @license: MPL 1.1/GPL 2.0/LGPL 2.1
*/

function getRotateElement(doc, create){
  var rotate = doc.getElementById("__rotate_body");
  if(rotate || !create){
    return rotate;
  }
  var body = doc.getElementsByTagName("body")[0];
  var div = doc.createElement("div");
  div.setAttribute("id", "__rotate_body");
  while(body.hasChildNodes()){
    div.appendChild(body.childNodes[0]);
  }
  body.appendChild(div);
  return div;
}

function getAngle(doc){
  var div = getRotateElement(doc, false);
  if(div){
    return div.angle;
  }else{
    return 0;
  }
}

function rotate(win, angle){
  var doc = win.document;
  var margin = 17; // scroll bar width
  if(getAngle(doc) == angle){
    return false;
  }
  var div = getRotateElement(doc, true);
  div.angle = angle;
  if(angle == -90 || angle == 90){
    var width = win.innerHeight - margin;
    var height = win.innerWidth - margin;
    var left;
    var top;
    if(angle == -90){
      left = (win.innerWidth - width) / 2;
      top = 1000 - win.innerWidth + left;
    }else{
      top = (win.innerHeight - height) / 2;
      left = 10000 - win.innerWidth - top;
    }
    div.setAttribute("style",
      "-moz-transform: rotate(" + angle + "deg);"
      + "position: absolute;"
      + "width: " + width + "px;"
      + "height: " + height + "px;"
      + "left: " + left + "px;"
      + "top: " + top + "px;"
    );
  }else if(angle == 180){
    div.setAttribute("style", "-moz-transform: rotate(180deg);");
  }else{
    div.removeAttribute("style");
  }
  return true;
}

function getScroll(win){
  let angle = getAngle(win.content.document);
  if(angle == -90){
    return win.scrollX;
  }else if(angle == 90){
    return win.scrollMaxX - win.scrollX;
  }else if(angle == 180){
    return win.scrollMaxY - win.scrollY;
  }else{
    return win.scrollY;
  }
}

function setScroll(win, scrollY){
  let angle = getAngle(win.content.document);
  if(angle == -90){
    win.scrollTo(scrollY, win.scrollMaxY);
  }else if(angle == 90){
    win.scrollTo(win.scrollMaxX - scrollY, 0);
  }else if(angle == 180){
    win.scrollTo(0, win.scrollMaxY - scrollY);
  }else{
    win.scrollTo(0, scrollY);
  }
}

function rotateAndScroll(angle) {
  let scrollY = getScroll(window);
  if(rotate(window, angle)) {
    setScroll(window, scrollY);
  }
}
